/*
This examples shows how to to something simple in a complex fashion.
It's point was to use C++ features to test the msp430-g++ compiler.
*/

#include "hardware.h"

class Subject;
class Observer;

typedef class Observer *ObserverPtr;
    
// ----- fixed length list
class Tuple {
public:
    unsigned int length;
    ObserverPtr * elements;

    Tuple(ObserverPtr * elements, unsigned int length) :
        length(length), elements(elements)
    {
    }
};

// ----- observer pattern base clases

/**
viewer
*/
class Observer {
public:
    virtual void update(Subject *subject);
};

/**
data model
*/
class Subject {
private:
    Tuple *observers;
public:
    Subject(Tuple *observers) :
        observers(observers)
    {
    }
    
    unsigned short value;

    virtual void notify() {
        Tuple    *obslist = observers;
        Observer *observer;
        for (unsigned int i = 0; i<observers->length; i++)
        {
            observer = (Observer *)(obslist->elements)[i];
            if (observer) {
                observer->update(this);
            }
        }
    }
};


// ----- knight-rider like LED play

/**
viewer implementation: displaying value on port (LED araray)
*/
class LedDisplay: public Observer {
private:
    void *port;
public:
    LedDisplay(void *port) :
        Observer(), port(port)
    {
    }
    
    void update(Subject *subject) {
        ((struct port_full_t *)(port))->outport = subject->value;
    }
};

/**
alternative viewer implementation: displaying inverted value
on port (LED araray)
*/
class InverseLedDisplay: public Observer {
private:
    void *port;
public:
    InverseLedDisplay(void *port) :
        Observer(), port(port)
    {
    }
    
    void update(Subject *subject) {
        ((struct port_full_t *)(port))->outport = ~subject->value;
    }
};

/**
data model: shift around bits, notify Observer on each change
*/
class SimpleShifter: public Subject {
private:
    unsigned short count;
public:
    SimpleShifter(Tuple *observer) :
        Subject(observer)
    {
        count = 0;
    }
    
    void next() {
        if ((count & 0xf) < 8) {
            value = (1<<(count&7));
        } else {
            value = (0x80>>(count&7));
        }
        count++;
        notify();
    }
};

/**
data model: shift around bits, notify Observer on each change
*/
class Shifter: public Subject {
private:
    unsigned short count;
public:
    Shifter(Tuple *observer) :
        Subject(observer)
    {
        count = 0;
    }
    
    void next() {
        value = (1<<(count&7)) | (0x80>>(count&7));
        count++;
        notify();
    }
};

/**
Delay function.
*/
void delay(unsigned int d) {
   unsigned int i;
   for (i = 0; i<d; i++) {
      nop();
      nop();
   }
}

/**
Main function with some blinking leds
*/
int main(void) {
    WDTCTL = WDTCTL_INIT;               //Init watchdog timer

    P1OUT  = P1OUT_INIT;                //Init output data of port1
    P2OUT  = P2OUT_INIT;                //Init output data of port2

    P1SEL  = P1SEL_INIT;                //Select port or module -function on port1
    P2SEL  = P2SEL_INIT;                //Select port or module -function on port2

    P1DIR  = P1DIR_INIT;                //Init port direction register of port1
    P2DIR  = P2DIR_INIT;                //Init port direction register of port2

    P1IES  = P1IES_INIT;                //init port interrupts
    P2IES  = P2IES_INIT;
    P1IE   = P1IE_INIT;
    P2IE   = P2IE_INIT;
    
    P1OUT = P2OUT = P3OUT = P4OUT = P5OUT = P6OUT = 0x00;
    P1DIR = P2DIR = P3DIR = P4DIR = P5DIR = P6DIR = 0xff;

    LedDisplay          disp1(&port1);
    InverseLedDisplay   disp2(&port2);
    InverseLedDisplay   disp3(&port3);
    LedDisplay          disp4(&port4);
    LedDisplay          disp5(&port5);
    LedDisplay          disp6(&port6);
    
    ObserverPtr obs1[4] = {&disp3, &disp4, &disp5, &disp6};
    Tuple obslist1(obs1, 4);
    
    ObserverPtr obs2[2] = {&disp1, &disp2};
    Tuple obslist2(obs2, 2);
    
    Shifter             shifter1(&obslist1);
    SimpleShifter       shifter2(&obslist2);
    while (1) {                         //main loop, never ends...
        shifter1.next();
        shifter2.next();
        delay(0x4fff);
    }
}
