/**
see README.txt for details.

Software SPI demo.

SPI protocol description. The SPI RX interrupt function implements
the protocol described below. Received commands are delegated to the
foreground for execution. This allows for long running commands (like
sampling ADC etc.)

Protocol control characters:
    0xff         IDLE
    0xfe         DONE, command complete
    0xfd         BUSY, command is beeing received or executed
    0xfc..0xf2   [reserved]
    0xf1         NACK, answer complete, failure
    0xf0         ACK, answer complete, ok

These characters must not be used in the data fields.

Basic command transfer:
    master->slave: cmd   [arg bytes] DONE  IDLE.....................
    slave->master: IDLE  BUSY............  [BUSY] [ans bytes] (N)ACK

The master starts with the command followed by data (optionaly).
the slave is sending 0xfd (BUSY). 0xfe (DONE) finishes the command and
initiates execution.
The slave can continue with 0xfd (BUSY) while processing the command.
The answer is sent when ready, if there is any, and the command
is finished by 0xf0 (ACK) or 0xf1 (NACK), indicating success or failure.

As some characters are reserved and must not be used in the data, it's
not possible to transmit raw binary data. These special characters have
to escaped or the entire data has to be encoded. This example expects
binary data as hex encoded string.

e.g. "6466"(hex) -> "@B"(binary)

(C) 2004 Chris Liechti <cliechti@gmx.net>
this is distributed under a free software license, see license.txt

http://mspgcc.sf.net
*/
#include "hardware.h"
#include "irqspi.h"
#include <string.h>


char spi_buffer[20];            ///< send receive buffer for commands
volatile int spi_answer_length; ///< length of the answer in spi_buffer
volatile int spi_answer_byte;   ///< answer code: SPI_X_ACK or SPI_X_NACK

//commands
volatile enum {
    CMD_IDLE,           //mandatory, used in the statemachine in IRQ_SPI_rx
    CMD_SETLED,         //LED control
    CMD_DELAY,          //delay test
    CMD_SET_FREQ,       //PWM base freq
    CMD_SET_PWM,        //set both PWM duty cycles
} spi_cmd;                      ///< received command code

//Protocol control characters
#define SPI_X_IDLE      0xff
#define SPI_X_DONE      0xfe
#define SPI_X_BUSY      0xfd
#define SPI_X_NACK      0xf1
#define SPI_X_ACK       0xf0

/** just a simple busy wait delay */
void delay(unsigned short d) {
    for (; d; d--) {
        nop();
    }
}

/** Convert hex digit to a number.  */
static unsigned int fromhex(char a) {
    if ((a >= '0') && (a <= '9')) {
        return a - '0';
    } else if ((a >= 'a') && (a <= 'f')) {
        return (a - 'a') + 10;
    } else {
        return 0;//invalid hex digit
    }
}

/** Convert number to a hex digit.  */
static int tohex(unsigned int nib) {
    if (nib < 10) {
        return '0' + nib;
    } else {
        return 'a' + nib - 10;
    }
}

/** decode hex pairs of letters to binary */
unsigned int unhexlify(const char *in, char *out, unsigned int max_outlen) {
    unsigned int length = 0;
    while (*in && (length < max_outlen)) {
        *out = fromhex(*in++) << 4;
        if (*in == 0) break;    //odd number of input characters
        *out |= fromhex(*in++);
        out++;
        length++;
    }
    return length;
}

/** encode binary data to hex digit pairs*/
unsigned int hexlify(const char *in, unsigned int in_length, char *out, unsigned int max_outlen) {
    unsigned int length = 0;
    while ((length/2 < in_length) && (length < (max_outlen - 3))) {
        *out++ = tohex((*in >> 4) & 0xf);
        *out++ = tohex((*in >> 0) & 0xf);
        in++;
        length+=2;
    }
    *out = '\0';
    return length;
}


interrupt(NOVECTOR) IRQ_SPI_rx(void) {
    static enum {SPI_IDLE, SPI_GETCMD, SPI_PROCESSING, SPI_ANSWER} mode;
    static int pos;
    
    //statemachine
    switch (mode) {
        case SPI_IDLE:
            //wait for a command
            spi_data_out = SPI_X_IDLE;
            if (spi_data_in != SPI_X_IDLE) {
                spi_data_out = SPI_X_BUSY;
                spi_cmd = spi_data_in;
                mode = SPI_GETCMD;
            }
            break;
            
        case SPI_GETCMD:
            //read command args, wait for DONE tag
            spi_data_out = SPI_X_BUSY;
            if (spi_data_in == SPI_X_DONE) {
                pos = 0;
                if (spi_cmd) {
                    //any command except CMD_IDLE
                    spi_answer_byte = 0;
                    _BIC_SR_IRQ(LPM4_bits); //wakeup foreground to process command
                    mode = SPI_PROCESSING;
                } else {
                    //CMD_IDLE
                    spi_answer_length = 0;
                    mode = SPI_ANSWER;
                    spi_answer_byte = SPI_X_ACK;
                }
            } else {
                if (pos < sizeof(spi_buffer)) {
                    spi_buffer[pos++] = spi_data_in;
                }
            }
            break;
        
        case SPI_PROCESSING:
            //while the command is processed, send IDLE
            //if spi_answer_byte != 0 signals that the processing is done
            spi_data_out = SPI_X_BUSY;
            if (spi_answer_byte) {
                mode = SPI_ANSWER;
            }
            break;
        
        case SPI_ANSWER:
            //send answer: first send buffer if spi_answer_length is not
            //zero, then send the spi_answer_byte to finish the command
            if (pos < spi_answer_length) {
                spi_data_out = spi_buffer[pos++];
            } else {
                pos = 0;
                spi_data_out = spi_answer_byte;
                mode = SPI_IDLE;
            }
            break;
    }
}


/**
Main function with hardware init and command handler.
*/
int main(void) {
    char buffer[10];
    
    WDTCTL = WDTCTL_INIT;               //Init watchdog timer

    P1OUT  = P1OUT_INIT;                //Init output data of port1
    P1SEL  = P1SEL_INIT;                //Select port or module -function on port1
    P1DIR  = P1DIR_INIT;                //Init port direction register of port1
    P1IES  = P1IES_INIT;                //init port interrupts
    P1IE   = P1IE_INIT;

    P2OUT  = P2OUT_INIT;                //Init output data of port2
    P2SEL  = P2SEL_INIT;                //Select port or module -function on port2
    P2DIR  = P2DIR_INIT;                //Init port direction register of port2
    P2IES  = P2IES_INIT;                //init port interrupts
    P2IE   = P2IE_INIT;
    CAPD   = CAPD_INIT;                 //Init input buffers on port2 pins

    BCSCTL1 = BCSCTL1_INIT;
    BCSCTL2 = BCSCTL2_INIT;
    DCOCTL  = DCOCTL_INIT;

    TACTL = TACTL_INIT;
    CCR0  = 0xffff;
    CCR1  = 0x8000;
    CCR2  = 0x8000;
    CCTL0 = OUTMOD_TOGGLE;
    CCTL1 = OUTMOD_RESET_SET;
    CCTL2 = OUTMOD_RESET_SET;
    TACTL |= MC_UPTO_CCR0;
    
    spi_init();                         //initialize software SPI
    eint();                             //enable interrupts

    while (1) {
        //wait in lowpower mode for commands
        LPM0;
        
        //decode received data from hex to binary
        unhexlify(spi_buffer, buffer, sizeof(buffer));
        
        //execute command
        switch (spi_cmd) {
            case CMD_SETLED:
                //switching the LED on or off
                if (spi_buffer[0]) LED_ON(); else LED_OFF();
                strcpy(spi_buffer, "led ok");
                spi_answer_length = 6;
                spi_answer_byte = SPI_X_ACK;
                break;
            
            case CMD_SET_FREQ:
                //set the base frequency
                CCR0 = *((unsigned short *)buffer);
                spi_answer_length = 0;
                spi_answer_byte = SPI_X_ACK;
                break;
            
            case CMD_SET_PWM:
                //set the PWM outputs
                CCR1 = ((unsigned short *)buffer)[0];
                CCR2 = ((unsigned short *)buffer)[1];
                spi_answer_length = 0;
                spi_answer_byte = SPI_X_ACK;
                break;

            case CMD_DELAY:
                //test for a command that takes a while to execute
                delay(0xffff);
                delay(0xffff);
                strcpy(spi_buffer, "dly ok");
                spi_answer_length = 6;
                spi_answer_byte = SPI_X_ACK;
                break;
            
            default:
                //unknown command
                strcpy(spi_buffer, "unk cmd");
                spi_answer_length = 7;
                spi_answer_byte = SPI_X_NACK;
                break;
        }
    }
}

