#include <io.h>
;******************************************************************************
;   MSP-FET430x110 Demo - Software SPI Interface TLC549 Set P1.0 if > 0.5*Vcc  
;
;   Description: Read a TLC549 ADC with reference to Vcc using software SPI and 
;   store the 8-bit digital code in ADCData(R11).  The TLC549 is sampled in a 
;   continous loop. If ADCData > 0.5*Vcc, P1.0 set, else reset. 
;   ACLK = n/a, MCLK = SMCLK = default DCO ~ 800k 
;   //** VCC must be at least 3v for TLC549 **//
; 
;                        MSP430F1121
;                     -----------------
;                 /|\|              XIN|-  
;                  | |                 | 
;                  --|RST          XOUT|-
;       TLC549       |                 |
;     ----------     |                 |
;    |        CS|<---|P2.0         P1.0|-->LED
;  ~>| AIN+  CLK|<---|P2.1             |
;    |        DO|--->|P2.3             |    
;
;
;   M.Buccini
;   Texas Instruments, Inc
;   January 2002
;   Adapted for mspgcc by Steve Underwood <steveu@coppice.org>
;*****************************************************************************

#define     ADCData  R11                  
#define     Counter  R12                  
            .equ     CS,0x01                    ; P2.0 - Chip Select
            .equ     CLK,0x02                    ; P2.1 - Clock
            .equ     DO,0x08                    ; P2.3 - Data Out

            .section    .text
RESET:      mov.w   #0x300,R1               ; Initialize 'x112x stack
StopWDT:    mov.w   #WDTPW+WDTHOLD,&WDTCTL  ; Stop Watchdog Timer
SetupP2:    mov.b   #CS,&P2OUT              ; /CS set, - P2.x reset
            bis.b   #CS+CLK,&P2DIR          ; /CS and CLK outputs
SetupP1:    bis.b   #0x01,&P1DIR            ; P1.0 output
                                            ;
Mainloop:   call    #Meas_549               ; Call subroutine
            bic.b   #0x01,&P1OUT            ; P1.0 = 0
            cmp.w   #0x7F,ADCData           ; ADCData > 0.5Vcc?
            jlo     Mainloop                ; Again
            bis.b   #0x01,&P1OUT            ; P1.0 = 1
            jmp     Mainloop                ; Again
                                            ;
;----------------------------------------------------------------------------- 
Meas_549:   ;Subroutine to read TLC549, data is shifted into ADCData 
;           (R11), Counter (R12) is used as a bit counter. 
;----------------------------------------------------------------------------- 
            mov.w   #8,Counter              ; 8 data bits
            clr.w   ADCData                 ; Clear data buffer
            bic.b   #CS,&P2OUT              ; /CS reset, enable ADC
ADC_Loop:   bit.b   #DO,&P2IN               ; (4) DO -> C (carry)
            rlc.w   ADCData                 ; (1) C -> ADCData
            bis.b   #CLK,&P2OUT             ; (4) Clock high
            bic.b   #CLK,&P2OUT             ; (4) Clock low
            dec.w   Counter                 ; (1) All bits shifted in?
            jnz     ADC_Loop                ; (2) If not --> ADC_Loop
            bis.b   #CS,&P2OUT              ; /CS set, disable ADC 
            ret                             ; Return from subroutine
                                            ;
;----------------------------------------------------------------------------- 
;           Interrupt Vectors Used MSP430x11x1         
;----------------------------------------------------------------------------- 
            .section    .vectors
            .org    0x1E,0xFF               ; MSP430 RESET Vector
            .word   RESET                   ; 
